/*
    framework jex
    lexical analyzer
    author: Christoph Zenger
    date: 22.03.2000
*/ 

package jex;

import jaco.framework.parser.Symbol;

%%

%class Scanner
%implements Tokens
%function nextToken
%type Symbol

%eofval{
   return mkToken(EOF);
%eofval}

%{

/** returns a textual representation of the token
 */
    static String representation(int token) {
        switch(token) {
            case SEMI: return (";");
            case PLUS: return ("+");
            case MINUS: return ("-");
            case NUMLIT: return ("numlit");
            case EOF: return ("EOF");
            default: return ("<unknown> (" + token + ")");
        }
    }

/** returns the left position of the token just read 
  */
    protected int position() {        
        return Position.encode(yyline + 1, yychar - firstCharInLine);
    }

/** returns the right position of the token just read 
  */
    protected int rposition() {        
        return Position.encode(yyline + 1, 
            yychar - firstCharInLine + yylength() - 1);
    }

/** holds the character index of the first character in a line -1
  */    
    protected int firstCharInLine = -1;

/** builds a new token
  */
    protected Symbol mkToken(int token) {
        return new Symbol(token, position(), rposition());
    }

/** builds a new token, using obj as additional information
  */
    protected Symbol mkToken(int token, Object obj) {
        return new Symbol(token, position(), rposition(), obj);
    }
%}

%line
%char

LETTER          = [A-Za-z_]
DIGIT           = [0-9]
STRING_CHAR     = [^\n\\\"]

%%

";"             { return mkToken(SEMI); }
"+"             { return mkToken(PLUS); }
"-"             { return mkToken(MINUS); }

({DIGIT})+      { return mkToken(NUMLIT, new Integer(yytext())); }

[\ \t]+         { /* ignore white space. */ }

[\n\r\f]        { firstCharInLine = yychar; }
.               { Report.error(position(), "Illegal character: " + yytext()); }
